﻿namespace DependencyPropertySerialization
{
    using System;
    using System.Windows;
    using System.Runtime.Serialization;

    [Serializable]
    public class GraphNode : GraphNodeBase
    {
        #region Constructors
        public GraphNode(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            // Empty, but required.
        }

        public GraphNode(GraphNodeBase _parent)
            : base(_parent)
        {
            mSomeReadOnlyMember = 1234;
        }
        #endregion

        #region Dependency Properties
        /// <summary>
        /// A DependencyProperty marked as NonSerialized
        /// </summary>
        [NonSerialized]
        public static DependencyProperty DescriptionProperty = DependencyProperty.Register(
          "Description", typeof(string), typeof(GraphNode), new PropertyMetadata(String.Empty));

        public string Description
        {
            get { return (string)GetValue(DescriptionProperty); }
            set { SetValue(DescriptionProperty, value); }
        }

        public static DependencyProperty NameProperty = DependencyProperty.Register(
          "Name", typeof(string), typeof(GraphNode), new PropertyMetadata(String.Empty));

        public string Name
        {
            get { return (string)GetValue(NameProperty); }
            set { SetValue(NameProperty, value); }
        }
        #endregion

        #region Event-Test
        public delegate void OnNameChangedHandler(GraphNode _sender);

        // Multicast delegate
        public event OnNameChangedHandler OnNameChanged;
        #endregion

        /// <summary>
        /// Automatic CLR-Property
        /// </summary>
        public string AutoProperty
        {
            get;
            set;
        }

        /// <summary>
        /// A private member marked as NonSerialized
        /// </summary>
        [NonSerialized]
        public int mNonSerializedField;

        /// <summary>
        /// Backing field whose associated property can only be read
        /// </summary>
        private int mSomeReadOnlyMember;

        /// <summary>
        /// Read-Only CLR Property
        /// </summary>
        public int SomeReadOnlyProperty
        {
            get { return mSomeReadOnlyMember; }
        }

        public override string ToString()
        {
            return String.Format("{0}:{1}:{2}:{3}:{4}", Name, Description, AutoProperty, mSomeReadOnlyMember, mNonSerializedField);
        }

        #region Helper Methods
        public void PrintTree()
        {
            PrintTreeRecursive(0);
        }

        private void PrintTreeRecursive(int _depth)
        {
            for (int i = 0; i < _depth; ++i)
            {
                System.Console.Write("\t");
            }

            System.Console.WriteLine( ToString() );

            foreach (GraphNode childRover in mChildren)
            {
                childRover.PrintTreeRecursive(_depth + 1);
            }
        }
        #endregion
    }
}
