﻿namespace WPFDependencyPropertySerializationError
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Diagnostics;
    using System.Reflection;
    using System.Runtime.Serialization;
    using System.Windows;
    using DependencyPropertySerialization.Shared;

    public struct DependencyValueMap
    {
        public DependencyValueMap(DependencyProperty _dp, DependencyObject _obj, object _value)
        {
            dp = _dp;
            value = _value;
            obj = _obj;
        }

        public DependencyObject obj;
        public DependencyProperty dp;
        public object value;
    }

    public sealed class ReflectionSerializationSurrogate : ISerializationSurrogate
    {
        #region ISerializationSurrogate Members
        public void GetObjectData(object obj, SerializationInfo info, StreamingContext context)
        {
            PropertyDescriptorCollection descriptors = TypeDescriptor.GetProperties(obj.GetType(),
                    new Attribute[] { new PropertyFilterAttribute(PropertyFilterOptions.SetValues | 
                                                                  PropertyFilterOptions.UnsetValues | 
                                                                  PropertyFilterOptions.Valid ) });

            List<FieldInfo> fieldInformation = ReflectionHelper.GetSerializableFieldInformation(obj.GetType());

            Debug.Print("SerializableDependencyObject.GetObjectData invoked.");
            Debug.Print("Serializing object of type: {0}", obj.GetType().Name);
            foreach (FieldInfo fiRover in fieldInformation)
            {
                Debug.Print("\tSerializing member '{0}'", fiRover.Name);

                info.AddValue(fiRover.Name, fiRover.GetValue(obj));
            }

            DependencyObject dependencyObject = obj as DependencyObject;

            if (null != dependencyObject)
            {
                foreach (PropertyDescriptor propertyDescriptor in descriptors)
                {
                    if (!DependencyPropertyHelper.IsSerializableDependencyProperty(dependencyObject, propertyDescriptor))
                        continue;

                    Debug.Print("\tSerializing property '{0}'", propertyDescriptor.Name);
                    info.AddValue(propertyDescriptor.Name, propertyDescriptor.GetValue(obj));
                }
            }
        }

        public object SetObjectData(object obj, SerializationInfo info, StreamingContext context, ISurrogateSelector selector)
        {
            Type thisType = obj.GetType();
            Debug.Print("SerializableDependencyObject deserialization constructor invoked.");
            Debug.Print("Deserializing object of type: {0}", thisType.Name);

            DependencyObject dependencyObject = obj as DependencyObject;
            
            // Setting the data works with this one... But the formatter will not re-stitch the 
            // object graph..
            DependencyObject doTest2 = (DependencyObject)Activator.CreateInstance(thisType);

            // De-Serialize Fields
            List<FieldInfo> fieldInformation = ReflectionHelper.GetSerializableFieldInformation(thisType);

            foreach (FieldInfo fiRover in fieldInformation)
            {
                fiRover.SetValue(obj, info.GetValue(fiRover.Name, fiRover.FieldType));
            }

            if (null == dependencyObject)
                return obj;

            //
            // De-Serialize DependencyProperties
            //
            PropertyDescriptorCollection descriptors = TypeDescriptor.GetProperties(thisType,
                    new Attribute[] { new PropertyFilterAttribute(PropertyFilterOptions.SetValues | 
                                                                  PropertyFilterOptions.UnsetValues | 
                                                                  PropertyFilterOptions.Valid ) });

            List<DependencyValueMap> mappingList = new List<DependencyValueMap>();

            foreach (PropertyDescriptor propertyDescriptor in descriptors)
            {
                if (!DependencyPropertyHelper.IsSerializableDependencyProperty(dependencyObject, propertyDescriptor))
                    continue;

                if (propertyDescriptor.Attributes[typeof(NonSerializedAttribute)] == null)
                {
                    Debug.Print("\tDeserializing dependency property '{0}'", propertyDescriptor.Name);
                    DependencyProperty dp = DependencyPropertyHelper.FindDependencyProperty(obj, propertyDescriptor.Name);

                    if (null != dp)
                    {
                        mappingList.Add(new DependencyValueMap(dp, obj as DependencyObject, info.GetValue(propertyDescriptor.Name, propertyDescriptor.PropertyType)));
                    }
                    else
                    {
                        throw new SerializationException(String.Format(@"Failed to deserialize property '{0}' on object of type '{1}'. 
                    Property could not be found. Version Conflict?", propertyDescriptor.Name, thisType));
                    }
                }
            }
            
            foreach (DependencyValueMap dvRover in mappingList)
            {
                // doTest2.SetValue(dvRover.dp, dvRover.value);
                dependencyObject.SetValue(dvRover.dp, dvRover.value);
            }

            // obj = doTest2;
            return dependencyObject;
        }

        #endregion
    }
}
